

#    MIDAS Access Client  for the  SM  (LyrTech Signal Master) service

global env
source [file join $env(MIDASBASE) SOAP tcl MIDASAccessClient.tcl]    ;#   load default support procedures

global SMHost SMPort

if {[info exists SMHost] == 0} then {set SMHost localhost}
if {[info exists SMPort] == 0} then {set SMPort 8015}

variable uri "urn:SM"
variable proxy "http://$SMHost:$SMPort/SM"

proc SMClient__SetUpTemp {} {
   global env tcl_platform

   if {![info exists env(MIDASTempDir)] || ![file exists $env(MIDASTempDir)]} then {
       switch $tcl_platform(platform) {
          windows {set env(MIDASTempDir) [file join c:/ temp]}
          default {set env(MIDASTempDir) \tmp}
       }
   }

   catch {file mkdir [file join $env(MIDASTempDir) SMClient]}
}

proc SMClient__SMUpLoad {file} {
    global env

    SMClient__SetUpTemp

    set path [file join $env(MIDASTempDir) SMClient [file tail $file].base64]

    set z [catch {exec base64 --encode $file $path} m]
    if {$z != 0} then {midas-warning "Error in base64 encoding $file in SMUpLoad\n$m"; return}

    set z [catch {open $path r} f]
    if {$z != 0} then {midas-warning "Cannot open file $path\n$f"; return}
    set contents [read $f]
    close $f

    SMClient__SMUpLoad_ $file $contents
}

SOAP::create SMClient__SMUpLoad_ -proxy $proxy -uri $uri -timeout 20000 \
     -name SMUpLoad \
     -params {file string contents strings}  \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SMClient__SMInit -proxy $proxy -uri $uri -timeout 5000 \
     -name SMInit \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply 

SOAP::create SMClient__SMGetMode -proxy $proxy -uri $uri -timeout 5000 \
     -name SMGetMode \
     -params {id int}  \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SMClient__SMGetSMType -proxy $proxy -uri $uri -timeout 5000 \
     -name SMGetSMType \
     -params {id int}  \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SMClient__SMGetVersion -proxy $proxy -uri $uri -timeout 5000 \
     -name SMGetVersion \
     -params {id int component int}  \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SMClient__SMGetUsers -proxy $proxy -uri $uri -timeout 5000 \
     -name SMGetUsers \
     -params {id int}  \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply 

SOAP::create SMClient__SMSetUser -proxy $proxy -uri $uri -timeout 5000 \
     -name SMSetUser \
     -params {id int user string}  \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply 

SOAP::create SMClient__SMSetLock -proxy $proxy -uri $uri -timeout 5000 \
     -name SMSetLock \
     -params {id int mode int}  \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply 

SOAP::create SMClient__SMGetLock -proxy $proxy -uri $uri -timeout 5000 \
     -name SMGetLock \
     -params {id int}  \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply 

SOAP::create SMClient__SMGetDSPRead -proxy $proxy -uri $uri -timeout 5000 \
     -name SMGetDSPRead \
     -params {id int offset int len int}  \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply 

SOAP::create SMClient__SMSetDSPWrite -proxy $proxy -uri $uri -timeout 5000 \
     -name SMSetDSPWrite \
     -params {id int offset int len int data string}  \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply 

SOAP::create SMClient__SMSetFPGALoad -proxy $proxy -uri $uri -timeout 10000 \
     -name SMSetFPGALoad \
     -params {id int bitstream string}  \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply 

SOAP::create SMClient__SMSetDSPLoad -proxy $proxy -uri $uri -timeout 10000 \
     -name SMSetDSPLoad \
     -params {id int select int format int filename string}  \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply 

SOAP::create SMClient__SMExit -proxy $proxy -uri $uri -timeout 5000 \
     -name SMExit \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SMClient__SMVerbose -proxy $proxy -uri $uri -timeout 5000 \
     -name SMVerbose \
     -params {opt int} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply 



package provide SMClient 1.0
