#    MIDAS Access Server  for  Spectrum 

global env
source [file join $env(MIDASBASE) TclHttpd tcl MIDASDomainHandler.tcl]       ;#  load default support procedures

global MIDAS_SpectrumAccessServer_Verbose
set    MIDAS_SpectrumAccessServer_Verbose 1

global MIDAS_SOAPDomainHandler_Verbose
set MIDAS_SOAPDomainHandler_Verbose 0

package require Spectrum

# Use namespaces to isolate your methods
namespace eval urn:SpectrumServer {
  catch {namespace import ::Spectrum::Debug}
  catch {namespace import ::Spectrum::Logging}
  catch {namespace import ::Spectrum::New1D}
  catch {namespace import ::Spectrum::New2D}
  catch {namespace import ::Spectrum::Delete}
  catch {namespace import ::Spectrum::Zero}
  catch {namespace import ::Spectrum::Details}
  catch {namespace import ::Spectrum::Names}
  catch {namespace import ::Spectrum::Read1D}
  catch {namespace import ::Spectrum::Read2D}
  catch {namespace import ::Spectrum::Write1D}
  catch {namespace import ::Spectrum::Write2D}
  catch {namespace import ::Spectrum::Title}
  catch {namespace import ::Spectrum::Label}
  catch {namespace import ::Spectrum::Info}
  catch {namespace import ::Spectrum::Calib}
  catch {namespace import ::Spectrum::Compressions}
  catch {namespace import ::Spectrum::Error}

  proc SpectrumServerVerbose {option} {
        global MIDAS_SpectrumAccessServer_Verbose
        global MIDAS_SOAPDomainHandler_Verbose
        set MIDAS_SpectrumAccessServer_Verbose $option
        set MIDAS_SOAPDomainHandler_Verbose $option
        set z [catch {Logging $option} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecVerbose detected error in call to Spectrum::Logging"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
  }

    proc SpecDebug {args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecDebug with parameter $args"}
        set z [catch {Debug $args} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecDebug detected error in call to Spectrum::Debug"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecLogging {args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecLogging with parameter $args"}
        set z [catch {Logging $args} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecLogging detected error in call to Spectrum::Logging"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecNew {n b r args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecNew with parameters $n $b $r $args,(len: [llength $args])"}
        if {[llength $args]==0 || [second $args]==0} {
          set z [catch {New1D $n $b $r} m]
          if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecNew detected error in call to Spectrum::New1D"}
        } else {
          set z [catch {New2D $n $b $r [first $args] [second $args]} m]
          if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecNew detected error in call to Spectrum::New2D"}
        }

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecNew1D {n b r} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecNew1D with parameters $n $b $r"}
        set z [catch {New1D $n $b $r} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecNew1D detected error in call to Spectrum::New1D"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecNew2D {n b0 r0 b1 r1} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecNew2D with parameters $n $b0 $r0 $b1 $r1"}
        set z [catch {New2D $n $b0 $r0 $b1 $r1} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecNew2D detected error in call to Spectrum::New2D"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecDelete {n} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecDelete with parameter $n"}
        set z [catch {Delete $n} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecDelete detected error in call to Spectrum::Delete"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecZero {n} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecZero with parameter $n"}
        set z [catch {Zero $n} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecZero detected error in call to Spectrum::Zero"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecDetails {n} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecDetails with parameter $n"}
        set z [catch {Details $n} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecDetails detected error in call to Spectrum::Details"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecNames {args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecNames with parameter $args"}
        set z [catch {Names $args} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecNames detected error in call to Spectrum::Names"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecRead1D {n args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecRead1D with parameters $n $args"}
        if {[llength $args] < 2} {
           set z [catch {Details $n} d]
           if {[llength $args] == 1} {
             set b [lindex $args 0]
             set r [expr [lindex $d 1] + [lindex $d 2] - $b]
           } else {
             set b [lindex $d 1]
             set r [lindex $d 2]
           }
         } else {
           set b [lindex $args 0]
           set r [lindex $args 1]
         }
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "calling method Spectrum::Read1D with parameters $n $b $r"}
        set z [catch {Read1D $n $b $r} d]
        if {$z != 0} then {return -code error -errorcode "$d" -errorinfo "SpecRead1D detected error in call to Spectrum::Read1D"} \
        else {
           set z [catch {::base64::encode $d} m]
           if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecRead1D detected error in call to ::base64::encode"}
        }

        set status "$m"
        return [rpcvar::rpcvar base64Binary $status]
    }

    proc SpecRead2D {n args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecRead2D with parameters $n $args"}
        if {[llength $args] < 4} {
           set z [catch {Details $n} d]
           if {[llength $args] == 0} {
             set b1 [lindex $d 1]
             set r1 [lindex $d 2]
             set b2 [lindex $d 3]
             set r2 [lindex $d 4]
           }
           if {[llength $args] == 1} {
             set b1 [lindex $args 0]
             set r1 [expr [lindex $d 1] + [lindex $d 2] - $b1]
             set b2 [lindex $d 3]
             set r2 [lindex $d 4]
           }
           if {[llength $args] == 2} {
             set b1 [lindex $args 0]
             set r1 [lindex $args 1]
             set b2 [lindex $d 3]
             set r2 [lindex $d 4]
           }
           if {[llength $args] == 3} {
             set b1 [lindex $args 0]
             set r1 [lindex $args 1]
             set b2 [lindex $args 2]
             set r2 [expr [lindex $d 3] + [lindex $d 4] - $b2]
           }
         } else {
           set b1 [lindex $args 0]
           set r1 [lindex $args 1]
           set b2 [lindex $args 2]
           set r2 [lindex $args 3]
         }
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "calling method Spectrum::Read2D with parameters $n $b1 $r1 $b2 $r2"}
        set z [catch {Read2D $n $b1 $r1 $b2 $r2} d]
        if {$z != 0} then {return -code error -errorcode "$d" -errorinfo "SpecRead2D detected error in call to Spectrum::Read2D"} \
        else {
           set z [catch {::base64::encode $d} m]
           if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecRead2D detected error in call to ::base64::encode"}
        }

        set status "$m"
        return [rpcvar::rpcvar base64Binary $status]
    }

     proc SpecWrite1D {n c args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecWrite1D with parameters $n (counts) $args"}
        if {$MIDAS_SpectrumAccessServer_Verbose && [string length $c] < 256} {puts "counts $c"}
        if {[llength $args] < 2} {
           set z [catch {Details $n} d]
           if {[llength $args] == 1} {
             set b [lindex $args 0]
             set r [expr [lindex $d 1] + [lindex $d 2] - $b]
           } else {
             set b [lindex $d 1]
             set r [lindex $d 2]
           }
         } else {
           set b [lindex $args 0]
           set r [lindex $args 1]
         }
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "calling method Spectrum::Write1D with parameters $n $b $r (counts)"}
        set z [catch {::base64::decode $c} d]
        if {!$z} {set z [catch {Write1D $n $b $r $d} m]
        } else {set m $d}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecWrite2D {n c args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecWrite2D with parameters $n (counts) $args"}
        if {[llength $args] < 4} {
           set z [catch {Details $n} d]
           if {[llength $args] == 0} {
             set b1 [lindex $d 1]
             set r1 [lindex $d 2]
             set b2 [lindex $d 3]
             set r2 [lindex $d 4]
           }
           if {[llength $args] == 1} {
             set b1 [lindex $args 0]
             set r1 [expr [lindex $d 1] + [lindex $d 2] - $b1]
             set b2 [lindex $d 3]
             set r2 [lindex $d 4]
           }
           if {[llength $args] == 2} {
             set b1 [lindex $args 0]
             set r1 [lindex $args 1]
             set b2 [lindex $d 3]
             set r2 [lindex $d 4]
           }
           if {[llength $args] == 3} {
             set b1 [lindex $args 0]
             set r1 [lindex $args 1]
             set b2 [lindex $args 2]
             set r2 [expr [lindex $d 3] + [lindex $d 4] - $b2]
           }
         } else {
           set b1 [lindex $args 0]
           set r1 [lindex $args 1]
           set b2 [lindex $args 2]
           set r2 [lindex $args 3]
         }
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "calling method Spectrum::Write2D with parameters $n $b1 $r1 $b2 $r2 (counts)"}
        set z [catch {::base64::decode $c} d]
        if {!$z} {set z [catch {Write2D $n $b1 $r1 $b2 $r2 $d} m]
        } else {set m $d}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecGetTitle {s} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecGetTitle with parameters $s"}
        set z [catch {Title $s} m]

        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecGetTitle detected error in call to Spectrum::Title"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecPutTitle {s t} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecPutTitle with parameters $s $t"}
        set z [catch {Title $s $t} m]

        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecPutTitle detected error in call to Spectrum::Title"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecGetLabel {s} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecGetLabel with parameters $s"}
        set z [catch {Label $s} m]

        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecGetLabel detected error in call to Spectrum::Label"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecPutLabel {s l} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecPutLabel with parameters $s $l"}
        set z [catch {Label $s $l} m]

        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecPutLabel detected error in call to Spectrum::Label"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecGetInfo {s n} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecGetInfo with parameters $s $n"}
        set z [catch {Info $s $n} m]

        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecGetInfo detected error in call to Spectrum::Info"}

        set status "$m"
        return [rpcvar::rpcvar string $status]

    }

    proc SpecPutInfo {s n i} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecPutInfo with parameters $s $n $i"}
        set z [catch {Info $s $n $i} m]

        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecPutInfo detected error in call to Spectrum::Info"}

        set status "$m"
        return [rpcvar::rpcvar string $status]

    }

    proc SpecGetCalib {s d} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecGetCalib with parameters $s $d"}
        set z [catch {Calib $s $d} m]

        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecGetCalib detected error in call to Spectrum::Calib"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecPutCalib {s d c} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecPutCalib with parameters $s $d $c"}
        set z [catch {Calib $s $d $c} m]

        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecCalib detected error in call to Spectrum::Calib"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecCompressions {c1 c2} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecCompressions with parameters $c1 $c2"}
        set z [catch {Compressions $c1 $c2} m]

        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecCompressions detected error in call to Spectrum::Compressions"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecError {e} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "called method SpecError with parameter $e"}
        set z [catch {Error $e} m]

        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecError detected error in call to Spectrum::Error"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }


    # We have to publish the public methods...

     SOAP::export SpectrumServerVerbose
     SOAP::export SpecDebug
     SOAP::export SpecLogging
     SOAP::export SpecNew
     SOAP::export SpecNew1D
     SOAP::export SpecNew2D
     SOAP::export SpecDelete
     SOAP::export SpecZero
     SOAP::export SpecDetails
     SOAP::export SpecNames
     SOAP::export SpecRead1D
     SOAP::export SpecRead2D
     SOAP::export SpecWrite1D
     SOAP::export SpecWrite2D
     SOAP::export SpecGetTitle
     SOAP::export SpecPutTitle
     SOAP::export SpecGetLabel
     SOAP::export SpecPutLabel
     SOAP::export SpecGetInfo
     SOAP::export SpecPutInfo
     SOAP::export SpecGetCalib
     SOAP::export SpecPutCalib
     SOAP::export SpecCompressions
     SOAP::export SpecError
}

# register this service with tclhttpd
puts [SOAP::Domain::register  -prefix  /SpectrumServer -namespace urn:SpectrumServer -uri urn:SpectrumServer]


proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/SpectrumServer {optsname sock args} {   \
    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}


#       set z [catch {interp eval "" ::namespace inscope $options(-namespace) $methodName $argValues} msg]
#       if {[llength $msg]==2} {
#          append xml "<status xsi:type=\"xsd:[lindex [set msg0 [lindex $msg 0]] 4]\">[lindex $msg0 5]</status>"
#          append xml "<return xsi:type=\"xsd:[lindex [set msg1 [lindex $msg 1]] 4]\">[lindex $msg1 5]</return>"
#       } else {
#          append xml "<return xsi:type=\"xsd:[lindex $msg 4]\">[lindex $msg 5]</return>"
#       }

